#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "inc/crc_ecc.h"
#include "inc/emu_tran_file.h"
#include "inc/mfm_decoder.h"
#include "inc/msg.h"
// Read the data for a track into deltas.
//
// deltas: Delta transition data returned
// max_deltas: Size of deltas buffer in words
// cyl, head: Track deltas are for
// fn: Filename to read data from
// return: Number of delta words read
int read_track_deltas(char *fn, uint16_t deltas[], int max_deltas, int *cyl,
      int *head)
{
   static int fd = -1;
   int bytes;
   static uint32_t fileid;
   int rc;


   if (fd == -1) {
      fd = open(fn,O_RDONLY);
      if (fd < 0) {
         msg(MSG_FATAL,"open failed %s\n",strerror(errno));
         exit(1);
      }
      if (read(fd, &fileid, sizeof(fileid)) < sizeof(fileid)) {
         msg(MSG_FATAL,"Short read fileid\n");
         exit(1);
      }
      if (fileid != 1) {
         msg(MSG_FATAL,"Unexpected File ID word %d expected 1\n");
         exit(1);
      }
   }

   if ((rc = read(fd, &bytes, sizeof(bytes))) < sizeof(bytes)) {
      if (rc == 0) {
         return -1;
      } else {
         msg(MSG_FATAL,"Short read\n");
         exit(1);
      }
   } else {
      if (bytes / sizeof(deltas[0]) >= max_deltas) {
         msg(MSG_FATAL,"Too many deltas\n");
         exit(1);
      }
      if (read(fd, cyl, sizeof(*cyl)) < sizeof(*cyl)) {
         msg(MSG_FATAL,"Short read\n");
         exit(1);
      }
      if (read(fd, head, sizeof(*head)) < sizeof(*head)) {
         msg(MSG_FATAL,"Short read\n");
         exit(1);
      }
      if (read(fd, deltas, bytes) < bytes) {
         msg(MSG_FATAL,"Short read\n");
         exit(1);
      }
      return bytes / sizeof(deltas[0]);
   }
}
main(int argc, char *argv[])
{
   uint16_t deltas[200000];
   int cyl, head;
   int rc;
   int fd;
   
   if (argc < 6) {
      printf("Usage: filename cyl head cmdline note\n");
   }
   fd = tran_file_write_header("/tmp/new_tran.raw", atoi(argv[2]), atoi(argv[3]),
      argv[4], argv[5], 0);

   rc = 0;
   while (rc >= 0) {
      rc = read_track_deltas(argv[1], deltas, sizeof(deltas)/2, &cyl,
        &head);
        int i;
        for (i = 0; i < rc; i++) {
           deltas[i] += 1;
        }
//printf("rc %d cyl %d head %d\n",rc, cyl, head);
      if (rc >= 0) {
         tran_file_write_track_deltas(fd, deltas, rc, cyl, head);
      }
   }
   tran_file_close(fd, 1);
}
